import { Http, Metadata } from '@zeedhi/core';
import { Form, Frame, Modal, ModalService, Tabs } from '@zeedhi/common';
import { TekGrid, TekRestDatasource, TekTreeGrid } from '@zeedhi/teknisa-components-common';

export class ScoreReportController {
    private addModal!: Modal;

    public openAddModal() {
        this.createScoreReportModal();
        this.openScoreReportModal();
    }

    public async createScoreReportModal() {
        this.addModal = await ModalService.createFromJson('scoreReport/modal/scoreReportDetail', true);
      }

    public destroyScoreReportModal() {
        this.addModal?.destroy();
    }

    public openScoreReportModal() {
        this.addModal.show();
    }

    public closeScoreReportModal() {
        this.addModal.hide();
    }

	public scoreReportGridIsVisible: boolean = true;

	public contractsGridIsVisible: boolean = true;

	get scoreReportDetailIsVisible() {
		return !this.scoreReportGridIsVisible;
	}

	public clickBackButtonScoreReport() {
		this.scoreReportGridIsVisible = true;
	}

	public clickBackButtonContracts() {
		this.contractsGridIsVisible = true;
	}

	public scoreReportGridCurrentRow: any = {};

	public contractGridCurrentRow: any = {};

	public scoreReportDetailHeaderTitle = '';

	public loadingScoreReportFinancialIsVisible = false;

	public loadingScoreReportCallIsVisible = false;

	public financialGridCurrentRow: any = {};

	public financialGridIsVisible: boolean = true;

	public NRPARCNEGOCIO: any = '';

	public firstFinancialGrid: boolean = true;

	public chamadosGridCurrentRow: any = {};

	public chamadosGridIsVisible: boolean = true;

	public produtosGridCurrentRow: any = {};

	get financialDetailIsVisible() {
		return !this.financialGridIsVisible;
	}

	get contractsDetailIsVisible() {
		return !this.contractsGridIsVisible;
	}

	public clickBackButtonFinancial() {
		this.financialGridIsVisible = true;
	}

	public static getInstanceTekGrid(name: string): TekGrid {
        return Metadata.getInstance<TekGrid>(name);
    }

	public async organizationScoreRowClick({ row } : any) {
		const frame = Metadata.getInstance<Frame>('scoreReportFrame');
		const organizationForm = Metadata.getInstance<Form>('organizationForm');
		const operadorClienteForm = Metadata.getInstance<Form>('operadorClienteForm');
		const operadorForm = Metadata.getInstance<Form>('operadorForm');
		const financeiroForm = Metadata.getInstance<Form>('financeiroForm');
		const chamadosForm = Metadata.getInstance<Form>('chamadosForm');
		const ScoreReportTab = Metadata.getInstance<Tabs>('ScoreReportTab');
		this.scoreReportGridCurrentRow = row;
		this.scoreReportDetailHeaderTitle = row.NMORGANIZACAO;
		this.scoreReportGridIsVisible = false;
		frame.isVisible = true;
		organizationForm.value = row;
		operadorClienteForm.value = row;
		operadorForm.value = row;
		this.loadingScoreReportFinancialIsVisible = true;
		Http.get('/getContratosForScore', { params: row }).then((response) => {
			financeiroForm.value = response.data[0];
			this.loadingScoreReportFinancialIsVisible = false;
		}).catch(() => {
			this.loadingScoreReportFinancialIsVisible = false;
		});

		this.loadingScoreReportCallIsVisible = true;
		Http.get('/getCallsForScore', { params: row }).then((responseCall) => {
			chamadosForm.value = responseCall.data;
			this.loadingScoreReportCallIsVisible = false;
		}).catch(() => {
			this.loadingScoreReportCallIsVisible = false;
		});
		// fix bug beforeChangeTab
		const params = {
			tabIndex: <Number>ScoreReportTab.activeTab,
		};
		this.beforeChangeScoreReportTab(params);
    }

	public getColumnStyle(params: any) {
		return params.row.IDPERCPIMP === 'B' ? 'om-next-grid-column-bold' : '';
    }

	public getProductStyle(params: any) {
		let cssclass = '';
		if (params.row.ATIVO === 'P') {
			cssclass = 'om-next-grid-bold';
		} else if (params.row.ATIVO === 'N') {
			cssclass = 'om-next-product-inactive';
		} else {
			cssclass = 'om-next-product-bold';
		}
		return cssclass;
    }

	public getStyleFilter(params: any) {
		let cssclass = '';
		if (params.row.VALIDATOR === 'V') {
			cssclass = 'row-vermelho';
		} else if (params.row.VALIDATOR === 'P') {
			cssclass = 'row-verde';
		}

		if (params.row.STATUS === 'BAIXADO') {
			cssclass += ' om-next-inactive';
		}
		return cssclass;
    }

	public beforeChangeScoreReportTab({ tabIndex }: any) {
		setTimeout(() => {
			if (tabIndex === 2) {
				const financialGrid = Metadata.getInstance<TekGrid>('financialGrid');
				financialGrid.datasource.filter.NRPARCNEGOCIO = this.scoreReportGridCurrentRow.NRPARCNEGOCIO;
				if (financialGrid.datasource instanceof TekRestDatasource) {
					financialGrid.datasource.dynamicFilter = {
						IDSTATBREC: [
							{
								operation: 'IN',
								relation: 'AND',
								value: ['E', 'A', 'P'],
							},
						],
					};
				}
				financialGrid.datasource.get();
			} else if (tabIndex === 5) {
				this.setChamados();
			} else if (tabIndex === 4) {
				this.setProdutos();
			} else if (tabIndex === 3) {
				this.setContratos();
			}
		}, 1000);
	}

	public beforeChangeContractTab({ tabIndex }: any) {
		setTimeout(() => {
			if (tabIndex === 1) {
				const itemsGridContract = Metadata.getInstance<TekTreeGrid>('itemsGridContract');
				itemsGridContract.datasource.filter.NRORG = this.contractGridCurrentRow.NRORG;
				itemsGridContract.datasource.filter.NRSEQCONTRATO = this.contractGridCurrentRow.NRSEQCONTRATO;
				itemsGridContract.datasource.filter.NRSEQORCAMENTO = this.contractGridCurrentRow.NRSEQORCAMENTO;
				itemsGridContract.datasource.get();
			} else if (tabIndex === 2) {
				this.setElementsContract();
			}
		}, 1000);
	}

	public async financeiroRowClick({ row }: any) {
		const frame = Metadata.getInstance<Frame>('financialFrame');
		this.financialGridCurrentRow = row;
		this.financialGridIsVisible = false;
		const financialForm = Metadata.getInstance<Form>('financialForm');
		const NFForm = Metadata.getInstance<Form>('NFForm');
		const installmentForm = Metadata.getInstance<Form>('installmentForm');
		frame.isVisible = true;
		financialForm.value = row;
		NFForm.value = row;
		installmentForm.value = row;
	}

	public async contractsRowClick({ row }: any) {
		this.contractGridCurrentRow = row;
		this.contractsGridIsVisible = false;
	}

	public async setChamados() {
		const chamadosGrid = Metadata.getInstance<TekGrid>('chamadosGrid');
		chamadosGrid.datasource.filter.ORG_ID = this.scoreReportGridCurrentRow.NRORG;
			if (chamadosGrid.datasource instanceof TekRestDatasource) {
				chamadosGrid.datasource.dynamicFilter = {
					STATUS: [
						{
							operation: 'IN',
							relation: 'AND',
							value: ['O', 'P', 'W'],
						},
					],
				};
			}
		chamadosGrid.datasource.get();
	}

	public async setProdutos() {
		const produtosGrid = Metadata.getInstance<TekGrid>('produtosGrid');
		produtosGrid.datasource.filter.NRPARCNEGOCIO = this.scoreReportGridCurrentRow.NRPARCNEGOCIO;
		if (produtosGrid.datasource instanceof TekRestDatasource) {
			produtosGrid.datasource.dynamicFilter = {
				ATIVO: [
					{
						operation: 'IN',
						relation: 'AND',
						value: ['P', 'S', 'N'],
					},
				],
			};
		}
		produtosGrid.datasource.get();
	}

	public async setContratos() {
		const contratosGrid = Metadata.getInstance<TekGrid>('contratosGrid');
		contratosGrid.datasource.filter.NRPARCNEGOCIO = this.scoreReportGridCurrentRow.NRPARCNEGOCIO;
		contratosGrid.datasource.get();
	}

	public async setElementsContract() {
		const elementsContractGrid = Metadata.getInstance<TekTreeGrid>('elementsContractGrid');
		elementsContractGrid.datasource.filter.NRORG = this.contractGridCurrentRow.NRORG;
		elementsContractGrid.datasource.filter.NRSEQORCAMENTO = this.contractGridCurrentRow.NRSEQORCAMENTO;
		elementsContractGrid.datasource.get();
	}
}
